/*
*	Process Type Helper for Arm
*
*	Jasperwang, Tencent inc.
*/
#include <linux/of.h>
#include <linux/string.h>
#include <linux/processor-type.h>
#include <linux/dmi.h>

#define MAGIC_NUM 0x5a5aa5a5

/* check phytium via dtb */
unsigned int  is_phytium2000(void)
{
	static int phytium2000_cpu = MAGIC_NUM;

	if (unlikely(phytium2000_cpu == MAGIC_NUM))
		phytium2000_cpu = of_machine_is_compatible("phytium,ft-2000a")
				|| of_machine_is_compatible("phytium,ft-2000ahk")
				|| of_machine_is_compatible("phytium,ft-2000plus")
				|| (dmi_name_in_processor_vendor("Phytium")
				&& dmi_name_in_socket_designation("FT2000"));

	return phytium2000_cpu;
}
EXPORT_SYMBOL(is_phytium2000);

/* check phytium via dtb */
unsigned int  is_phytium1500(void)
{
	static int phytium1500_cpu = MAGIC_NUM;

	if (unlikely(phytium1500_cpu == MAGIC_NUM))
		phytium1500_cpu = of_machine_is_compatible("phytium,ft-1500a")
				|| (dmi_name_in_processor_vendor("Phytium")
				&& dmi_name_in_socket_designation("FT1500"));

	return phytium1500_cpu;
}
EXPORT_SYMBOL(is_phytium1500);

/* check huawei via dmi info
 * notice: only available after subsys initcall !
*/
unsigned int  is_huawei(void)
{
	static int huawei_cpu = MAGIC_NUM;

	if (unlikely(huawei_cpu == MAGIC_NUM))
		huawei_cpu = dmi_name_in_vendors("HUAWEI")
				|| dmi_name_in_vendors("Huawei");

	return huawei_cpu;
}
EXPORT_SYMBOL(is_huawei);

unsigned int  is_virtual(void)
{
	static int virtual_cpu = MAGIC_NUM;

	if (unlikely(virtual_cpu == MAGIC_NUM))
		virtual_cpu = dmi_name_in_vendors("QEMU")
				|| dmi_match(DMI_PRODUCT_NAME, "KVM Virtual Machine");

	return virtual_cpu;
}
EXPORT_SYMBOL(is_virtual);
